// Generated by CoffeeScript 2.3.1
(function() {
  var assert, debug, default_music, pkg, tones, url;

  url = require('url');

  pkg = require('../../../package.json');

  this.name = `${pkg.name}:middleware:client:egress:post`;

  debug = (require('tangible'))(this.name);

  assert = require('assert');

  tones = require('../tones');

  default_music = tones.loop(tones.fr.waiting);

  this.include = async function() {
    var base, base1, base2, base3, base4, base5, pci, ref, ref1, ref2;
    if (((ref = this.session) != null ? ref.direction : void 0) !== 'egress') {
      return;
    }
    this.debug('Ready');
    if (!((this.session.ccnq_from_e164 != null) && (this.session.ccnq_to_e164 != null))) {
      this.debug('Missing e164 numbers');
      return this.respond('484');
    }
    // * session.e164_number (object) The doc.global_number record for the source of an outbound call.
    this.session.e164_number = (await this.cfg.prov.get(`number:${this.session.ccnq_from_e164}`).catch(function() {
      return {};
    }));
    await this.reference.add_in(this.session.e164_number._id);
    await this.user_tags(this.session.e164_number.tags);
    // * session.e164_number.fs_variables See doc.global_number.fs_variables
    // * doc.global_number (object, optional) Additional FreeSwitch variables to be set on egress calls (for the calling number). These will show up in CDRs on the client side.
    if (this.session.e164_number.fs_variables != null) {
      await this.set(this.session.e164_number.fs_variables);
    }
    // The URL module parses the SIP username as `auth`.

    // * hdr.P-Charge-Info Required for egress calls on the client side. A `403 No Charge-Info` SIP error is generated if it is not present. The username part is used to populate session.ccnq_account.
    pci = this.req.header('P-Charge-Info');
    if (pci != null) {
      this.session.ccnq_account = (url.parse(pci)).auth;
    } else {
      this.session.ccnq_account = (await this.reference.get_account());
    }
    if ((base = this.session).ccnq_account == null) {
      base.ccnq_account = null;
    }
    if (this.session.ccnq_account == null) {
      this.debug('Invalid Charge-Info', pci);
      return this.respond('403 Missing Charge-Info');
    }
    await this.reference.set_account(this.session.ccnq_account);
    await this.reference.add_in(`account:${this.session.ccnq_account}`);
    // Settings for calling number (see middleware/client/ingress/post.coffee.md):
    if (this.session.number.custom_music === true) {
      if ((base1 = this.session).music == null) {
        base1.music = [this.cfg.userdb_base_uri, this.session.number.user_database, 'voicemail_settings', 'music.wav'].join('/');
      }
    }
    if (typeof this.session.number.custom_music === 'string') {
      if ((base2 = this.session).music == null) {
        base2.music = [this.cfg.userdb_base_uri, this.session.number.user_database, 'voicemail_settings', this.session.number.custom_music].join('/');
      }
    }
    if ((this.session.country != null) && this.session.country in tones) {
      if ((base3 = this.session).music == null) {
        base3.music = tones.loop(tones[this.session.country].waiting);
      }
    }
    if ((base4 = this.session).music == null) {
      base4.music = this.cfg.music;
    }
    if ((base5 = this.session).music == null) {
      base5.music = default_music;
    }
    // Set parameters
    this.session.cdr_direction = this.session.direction;
    await this.set({
      // These are injected so that they may eventually show up in CDRs.
      ccnq_direction: this.session.direction,
      ccnq_account: this.session.ccnq_account,
      ccnq_profile: this.session.profile,
      ccnq_from_e164: this.session.ccnq_from_e164,
      ccnq_to_e164: this.session.ccnq_to_e164,
      // SIP parameters
      progress_timeout: 18,
      call_timeout: 300,
      effective_caller_id_number: this.session.ccnq_from_e164,
      sip_contact_user: this.session.ccnq_from_e164,
      sip_cid_type: 'pid',
      // Codec negotiation with late-neg:
      inherit_codec: (ref1 = this.session.inherit_codec) != null ? ref1 : true
    });
    if (this.session.ringback != null) {
      await this.set({
        ringback: this.session.ringback
      });
    }
    await this.export({
      t38_passthru: true,
      sip_wait_for_aleg_ack: (ref2 = this.session.wait_for_aleg_ack) != null ? ref2 : true,
      // Music
      hold_music: this.session.music
    });
    if (this.session.asserted != null) {
      await this.set({
        effective_caller_id_number: this.session.asserted
      });
    }
    if (this.session.dev_logger) {
      await this.reference.set_dev_logger(true);
    }
    this.report({
      state: 'client-egress-accepted',
      account: this.session.ccnq_account,
      number: this.session.e164_number._id,
      from_e164: this.session.ccnq_from_e164,
      to_e164: this.session.ccnq_to_e164
    });
    return this.debug('OK');
  };

}).call(this);
