// Generated by CoffeeScript 2.3.1
(function() {
  var pkg;

  pkg = require('../../../package');

  this.name = `${pkg.name}:middleware:client:ingress:fifo`;

  this.description = 'Maps an ingress call (which already went through the `local-number` middleware) to a FIFO/conference/menu.';

  this.include = async function() {
    var base, base1, base2, conf_name, item, items, m, music_uri, number, number_domain, ref, ref1, ref2, ref3, ref4, ref5, ref6, ref7, type;
    if (((ref = this.session) != null ? ref.direction : void 0) !== 'ingress') {
      return;
    }
    music_uri = (doc) => {
      if (doc.music == null) {
        return null;
      }
      return this.prompt.uri('prov', 'prov', doc._id, doc.music);
    };
    this.debug(`Testing for FIFO/conference/menu in ${this.destination}`);
    m = this.destination.match(/^(fifo|conf|menu|localconf)-(.+)$/);
    if (m == null) {
      return;
    }
    type = m[1];
    number = m[2];
    if (number == null) {
      return;
    }
    // The `localconf` type is used by other servers to transfer calls for conferences which are served on this server. (See [`../conference`](../conference.coffee.md).)
    // In this case the conference name is the number-domain and the conference name.
    if (type === 'localconf' && (this.session.number_domain_data == null)) {
      conf_name = number;
      m = conf_name.match(/^(.+)-([^-]+)$/);
      if (m == null) {
        this.debug.dev(`localconf ${m} is not properly formatted`);
        return;
      }
      type = 'conf';
      number_domain = m[1];
      number = m[2];
      this.session.number_domain = number_domain;
      this.session.number_domain_data = (await this.cfg.prov.get(`number_domain:${number_domain}`).catch((error) => {
        this.debug.dev(`number_domain ${number_domain}: ${error}`);
        return null;
      }));
      await this.user_tags((ref1 = this.session.number_domain_data) != null ? ref1.tags : void 0);
      if (((ref2 = this.session.number_domain_data) != null ? ref2.timezone : void 0) != null) {
        if ((base = this.session).timezone == null) {
          base.timezone = (ref3 = this.session.number_domain_data) != null ? ref3.timezone : void 0;
        }
      }
      if (((ref4 = this.session.number_domain_data) != null ? ref4.music : void 0) != null) {
        if ((base1 = this.session).music == null) {
          base1.music = music_uri(this.session.number_domain_data);
        }
      }
      if (((ref5 = this.session.number_domain_data) != null ? ref5.dialplan : void 0) != null) {
        if ((base2 = this.session).dialplan == null) {
          base2.dialplan = this.session.number_domain_data.dialplan;
        }
      }
    }
    // We won't be able to route if there is no number-domain data.
    if (this.session.number_domain_data == null) {
      this.debug.dev('Missing number_domain_data');
      return;
    }
    await this.reference.add_in(this.session.number_domain_data._id);
    // * doc.global_number.local_number To route to a FIFO, this field must contain `fifo-<fifo-number>@<number-domain>`. The fifo-number is typically between 0 and 9; it represents an index in doc.number_domain.fifos.
    // * doc.number_domain.fifos (array) An array describing the FIFOs in this number-domain, indexed on the fifo-number. Typically the fifo-number is from 0 to 9. See session.fifo for a description of the contents.
    // * doc.global_number.local_number To route to a menu, this field must contain `menu-<menu-number>@<number-domain>`. The menu-number is typically between 0 and 9; it represents an index in doc.number_domain.menus.
    // * doc.number_domain.menus (array) An array describing the menus in this number-domain, indexed on the menu-number. Typically the menu-number is from 0 to 9. See session.menu for a description of the contents.
    // * doc.global_number.local_number To route to a conference, this field must contain `conf-<conf-number>@<number-domain>`. The conf-number is typically between 0 and 9; it represents an index in doc.number_domain.conferences.
    // * doc.number_domain.conferences (array) An array describing the conferences in this number-domain, indexed on the conf-number. Typically the conf-number is from 0 to 9. See session.conf for a description of the contents.
    items = (function() {
      switch (type) {
        case 'fifo':
          return this.session.number_domain_data.fifos;
        case 'conf':
          return this.session.number_domain_data.conferences;
        case 'menu':
          return this.session.number_domain_data.menus; // Huh?
        default:
          return null;
      }
    }).call(this);
    if (items == null) {
      this.debug.csr(`Number domain has no data for ${type}.`);
      return;
    }
    // Move handling to `fifo` middleware.

    // * session.direction (string) If `fifo`, then the call is handled by a number-domain FIFO. See session.fifo.
    // * session.fifo (object) The element of doc.number_domain.fifos describing the current FIFO in use.
    if (!items.hasOwnProperty(number)) {
      this.debug.dev(`No property ${number} in ${type} of ${this.session.number_domain}.`);
      return;
    }
    item = items[number];
    if (item == null) {
      this.debug.csr(`Number domain has no data ${number} for ${type}.`);
      return;
    }
    // These are also found in middleware/client/egress/fifo.
    if (item.short_name == null) {
      item.short_name = `${type}-${number}`;
    }
    if (type === 'conf') {
      if (item.full_name == null) {
        item.full_name = `${this.session.number_domain}-${item.short_name}`;
      }
    } else {
      if (item.full_name == null) {
        item.full_name = `${item.short_name}@${this.session.number_domain}`;
      }
    }
    this.session[type] = item;
    this.direction(type);
    this.report({
      state: type,
      name: item.full_name,
      number: number != null ? number : null,
      number_domain: (ref6 = (ref7 = this.session.number_domain_data) != null ? ref7._id : void 0) != null ? ref6 : null
    });
    this.debug(`Using ${type} ${number}`, item);
  };

}).call(this);
