// Generated by CoffeeScript 2.3.1
(function() {
  var debug, pkg;

  pkg = require('../../../package.json');

  this.name = `${pkg.name}:middleware:client:ingress:local-number`;

  debug = (require('tangible'))(this.name);

  this.include = async function() {
    var base, music_uri, number, number_domain, ref, ref1, ref2, ref3, ref4, ref5, ref6, ref7;
    if (((ref = this.session) != null ? ref.direction : void 0) !== 'ingress') {
      return;
    }
    music_uri = (doc) => {
      if (doc.music == null) {
        return null;
      }
      return this.prompt.uri('prov', 'prov', doc._id, doc.music);
    };
    // Global number provides inbound routing
    // --------------------------------------

    // Now, we have two cases:
    // - either the global-number record contains the information we need to do translation (except for the source, which is why this module needs to be inserted between the national translations, and the Centrex routing modules), and we use that information;
    // - or it doesn't and we use a specific middleware to do the translation (based on the destination number, typically, to put that number in a specific `national` dialplan). The `france` module in the current directory illustrates this.

    // * doc.global_number.local_number (string) The identifier of the local-number into which this global-number gets translated for inbound calls. (The identifier must have the format `<number>@<number-domain>` and a `number:<number>@<number-domain>` record must exist.)
    if (((ref1 = this.session.e164_number) != null ? ref1.local_number : void 0) == null) {
      return;
    }
    await this.reference.add_in(`number:${this.session.e164_number.local_number}`);
    // These are used e.g. for Centrex, and override the destination number and number-domain.

    // Note: we still keep going through the national modules because we need source number translation from `e164` to `national`.
    this.debug('Using local_number', this.session.e164_number.local_number);
    // The dialplan and country (and other parameters) might also be available in the `number_domain:` record and should be loaded from there if the global-number does not specify them.

    // * session.number_domain (string) The number-domain of the current destination.
    // * session.number_domain_data (object) If present, the content of the `number_domain:<number-domain>` record for the current `session.number_domain`.
    // * doc.number_domain (object) Record describing a number-domain.
    // * doc.number_domain._id (required) `number_domain:<number-domain>`
    // * doc.number_domain.type (required) `number_domain`
    // * doc.number_domain.number_domain (required) `<number-domain>`
    [number, number_domain] = this.session.e164_number.local_number.split('@');
    this.destination = number;
    this.session.number_domain = number_domain;
    this.session.number_domain_data = (await this.cfg.prov.get(`number_domain:${number_domain}`).catch((error) => {
      var ref2;
      this.debug(`number_domain ${number_domain}: ${(ref2 = error.stack) != null ? ref2 : error}`);
      return {};
    }));
    await this.reference.add_in(this.session.number_domain_data._id);
    await this.user_tags(this.session.number_domain_data.tags);
    // * doc.number_domain.dialplan (optional) dialplan used for ingress calls to this domain.
    // * doc.number_domain.country (optional) country used for ingress calls to this domain.
    this.session.timezone = (ref2 = (ref3 = this.session.number_domain_data) != null ? ref3.timezone : void 0) != null ? ref2 : this.session.e164_number.timezone;
    this.session.music = music_uri(this.session.number_domain_data);
    if ((base = this.session).music == null) {
      base.music = music_uri(this.session.e164_number);
    }
    this.session.dialplan = (ref4 = (ref5 = this.session.number_domain_data) != null ? ref5.dialplan : void 0) != null ? ref4 : this.session.e164_number.dialplan;
    this.session.country = (ref6 = (ref7 = this.session.number_domain_data) != null ? ref7.country : void 0) != null ? ref6 : this.session.e164_number.country;
    if (this.session.country != null) {
      this.session.country = this.session.country.toLowerCase();
    }
    this.report({
      state: 'local-number',
      number: this.session.e164_number.local_number,
      number_domain: this.session.number_domain_data._id
    });
    this.debug('OK');
  };

}).call(this);
