// Generated by CoffeeScript 2.3.1
(function() {
  var Solid, Unique_ID, debug, default_eavesdrop_timeout, default_intercept_timeout, hand, make_id, pkg, send;

  pkg = require('../../../package.json');

  this.name = `${pkg.name}:middleware:client:ingress:send`;

  ({debug, hand} = (require('tangible'))(this.name));

  Solid = require('solid-gun');

  Unique_ID = 'Unique-ID';

  make_id = function() {
    return Solid.time() + Solid.uniqueness();
  };

  default_eavesdrop_timeout = 8 * 3600; // 8h

  default_intercept_timeout = 8 * 3600; // 8h

  this.include = function() {
    var ref, ref1;
    if (((ref = this.session) != null ? ref.direction : void 0) !== 'ingress') {
      return;
    }
    this.debug('Ready');
    // * session.initial_destinations (array) On ingress, client-side calls, list of target routes for `mod_sofia`, each route consisting of an object containing a `to_uri` URI (to be used with the standard SIP profile defined in session.sip_profile ) and an optional `parameters` array of per-leg parameter strings.

    // There might no `initial_destinations` if the call was rejected in ornaments, reject-anonymous, etc.
    if (((ref1 = this.session.initial_destinations) != null ? ref1.length : void 0) > 0) {
      return send.call(this, this.session.initial_destinations);
    }
  };

  // send
  // ====

  // Send call to (OpenSIPS or other) with processing for CFDA, CFNR, CFB.
  this.send = send = async function(destinations) {
    var base, cause, code, data, disposition, eavesdrop_key, eavesdrop_timeout, intercept_key, intercept_timeout, key, new_call, new_uuid, queuer, ref, ref1, ref2, ref3, ref4, res, sofia;
    // We must create a new call ID to differentiate the ingress call leg from the call leg towards
    // the agent (for screetching-eggs/monitor and black-metal).
    new_uuid = make_id();
    ({eavesdrop_timeout, intercept_timeout} = this.cfg);
    if (eavesdrop_timeout == null) {
      eavesdrop_timeout = default_eavesdrop_timeout;
    }
    if (intercept_timeout == null) {
      intercept_timeout = default_intercept_timeout;
    }
    key = `${this.destination}@${this.session.number_domain}`;
    intercept_key = `inbound_call:${key}`;
    await ((ref = this.local_redis) != null ? ref.setex(intercept_key, intercept_timeout, this.call.uuid) : void 0);
    this.session.agent = key;
    // Eavesdrop registration
    // ----------------------
    eavesdrop_key = `inbound:${key}`;
    ({queuer} = this.cfg);
    // Transfer-disposition values:
    // - `recv_replace`: we transfered the call out (blind transfer). (REFER To)
    // - `replaced`: we accepted an inbound, supervised-transfer call. (Attended Transfer on originating session.)
    // - `bridge`: we transfered the call out (supervised transfer).
    if (!(this.call.closed || this.session.dialplan !== 'centrex')) {
      this.debug('Set inbound eavesdrop', eavesdrop_key);
      await ((ref1 = this.local_redis) != null ? ref1.setex(eavesdrop_key, eavesdrop_timeout, new_uuid) : void 0);
      // The new call leg will always be bound to this agent.
      debug('CHANNEL_PRESENT', key, new_uuid);
      // Bind the agent to the call.
      if ((queuer != null) && (this.queuer_call != null)) {
        new_call = (await this.queuer_call(new_uuid));
        await queuer.set_agent(new_call, key);
      }
      // Note: inside the queuer, these calls are never pooled, so their state does not evolve.
      this.report({
        event: 'start-of-call',
        agent: key,
        call: new_uuid
      });
    }
    // Send the call(s)
    // ----------------
    sofia = destinations.map(({parameters = [], to_uri}, index) => {
      if (index === 0) {
        return `[origination_uuid=${new_uuid},${parameters.join(',')}]sofia/${this.session.sip_profile}/${to_uri}`;
      } else {
        return `[${parameters.join(',')}]sofia/${this.session.sip_profile}/${to_uri}`;
      }
    });
    this.debug('send', sofia);
    await this.set({
      continue_on_fail: true
    });
    this.debug('Bridging', sofia);
    this.report({
      state: 'ingress-bridging'
    });
    res = (await this.action('bridge', sofia.join(',')));
    await ((ref2 = this.local_redis) != null ? ref2.del(intercept_key) : void 0);
    // Post-attempt handling
    // ---------------------
    data = res.body;
    if ((base = this.session).bridge_data == null) {
      base.bridge_data = [];
    }
    this.session.bridge_data.push(data);
    // Retrieve the FreeSwitch Cause Code description, and the SIP error code.

    // ### last bridge hangup cause

    // For example: `NORMAL_CLEARING` (answer+bridge), `NORMAL_CALL_CLEARING` (bridge, call is over)
    cause = data != null ? data.variable_last_bridge_hangup_cause : void 0;
    // Note: there is also `variable_bridge_hangup_cause`

    // ### originate disposition

    // For example: `SUCCESS`, `ORIGINATOR_CANCEL`.
    if (cause == null) {
      cause = data != null ? data.variable_originate_disposition : void 0;
    }
    // Note: also `variable_endpoint_disposition` (`ANSWER`), `variable_DIALSTATUS` (`SUCCESS`)

    // ### last bridge proto-specific hangup cause

    // For example: `sip:200` → `200`
    code = data != null ? (ref3 = data.variable_last_bridge_proto_specific_hangup_cause) != null ? (ref4 = ref3.match(/^sip:(\d+)$/)) != null ? ref4[1] : void 0 : void 0 : void 0;
    // ### sip term status

    // For example: `200`
    if (code == null) {
      code = data != null ? data.variable_sip_term_status : void 0;
    }
    disposition = data != null ? data.variable_transfer_disposition : void 0;
    this.debug('Outcome', {cause, code, disposition});
    // ### For transfers, we also get state information.

    // - `variable_transfer_disposition: 'replaced'` (Attended Transfer on originating session)
    // - `variable_endpoint_disposition: 'ATTENDED_TRANSFER', 'BLIND_TRANSFER'
    this.session.was_connected = cause === 'NORMAL_CALL_CLEARING' || cause === 'NORMAL_CLEARING' || cause === 'SUCCESS';
    this.session.was_transferred = (data.variable_transfer_history != null) || data.variable_endpoint_disposition === 'BLIND_TRANSFER' || data.variable_endpoint_disposition === 'ATTENDED_TRANSFER';
    this.session.was_picked = cause === 'PICKED_OFF';
    // Success
    // -------

    // No further processing in case of success.
    if (this.session.was_connected) {
      this.debug(`Successful call when routing ${this.destination} through ${sofia.join(',')}`);
      this.notify({
        state: 'answered'
      });
      return;
    }
    if (this.session.was_transferred) {
      this.debug(`Transferred call when routing ${this.destination} through ${sofia.join(',')}`);
      this.notify({
        state: 'transferred'
      });
      return;
    }
    this.notify({
      event: 'missed'
    });
    if (this.session.was_picked) {
      this.debug(`Picked call when routing ${this.destination} through ${sofia.join(',')}`);
      this.notify({
        state: 'picked',
        from_agent: key
      });
      return;
    }
    // Note: we do not hangup since some centrex scenarios might want to do post-call processing (survey, ...).

    // Not Registered
    // --------------

    // OpenSIPS marker for not registered
    if (code === '604') {
      await this.debug.csr('not-registered', {
        destination: this.destination,
        enpoint: this.session.endpoint_name,
        tried_cfnr: this.session.tried_cfnr
      });
    }
    if (code === '604' && !this.session.tried_cfnr) {
      this.report({
        state: 'not-registered'
      });
      this.session.reason = 'unavailable'; // RFC5806
      if (this.session.cfnr_voicemail) {
        this.debug('cfnr:voicemail');
        this.destination = this.session.cfnr_voicemail_number;
        this.direction('voicemail');
        return;
      }
      if (this.session.cfnr_number != null) {
        this.debug('cfnr:forward');
        this.session.destination = this.session.cfnr_number;
        this.direction('forward');
        return;
      }
      if (this.session.cfnr != null) {
        this.debug('cfnr:fallback');
        this.session.tried_cfnr = true;
        return send.call(this, [
          {
            to_uri: this.session.cfnr
          }
        ]);
      }
    }
    // Try static routing on 604 without CFNR (or CFNR already attempted)
    if (code === '604') {
      if (this.session.fallback_destinations == null) {
        this.debug('cfnr: no fallback');
        return this.respond('500 No Fallback');
      }
      if (this.session.tried_fallback) {
        this.debug('cfnr: already attempted fallback');
        return this.respond('500 Fallback Failed');
      }
      this.debug('cfnr: fallback on 604');
      this.session.tried_fallback = true;
      return send.call(this, this.session.fallback_destinations);
    }
    // Busy
    // ----
    if (code === '486' && !this.session.tried_cfb) {
      this.report({
        state: 'user-busy'
      });
      this.session.reason = 'user-busy'; // RFC5806
      if (this.session.cfb_voicemail) {
        this.debug('cfb: voicemail');
        this.destination = this.session.cfb_voicemail_number;
        this.direction('voicemail');
        return;
      }
      if (this.session.cfb_number != null) {
        this.debug('cfb:number');
        this.session.destination = this.session.cfb_number;
        this.direction('forward');
        return;
      }
      if (this.session.cfb != null) {
        this.debug('cfb: fallback');
        this.session.tried_cfb = true;
        return send.call(this, [
          {
            to_uri: this.session.cfb
          }
        ]);
      }
    }
    // All other codes
    // ---------------
    this.debug(`Call failed: ${cause}/${code} when routing ${this.destination} through ${sofia.join(',')}`);
    // Use CFDA if present
    if (!this.session.tried_cfda) {
      this.report({
        state: 'no-answer'
      });
      this.session.reason = 'no-answer'; // RFC5806
      if (this.session.cfda_voicemail) {
        this.debug('cfda: voicemail');
        this.destination = this.session.cfda_voicemail_number;
        this.notify({
          state: 'forward-no-answer-to-voicemail'
        });
        this.direction('voicemail');
        return;
      }
      if (this.session.cfda_number != null) {
        this.debug('cfda:number');
        this.session.destination = this.session.cfda_number;
        this.notify({
          state: 'forward-no-answer-to-number'
        });
        this.direction('forward');
        return;
      }
      if (this.session.cfda != null) {
        this.debug('cfda: fallback');
        this.session.tried_cfda = true;
        this.notify({
          state: 'forward-no-answer'
        });
        return send.call(this, [
          {
            to_uri: this.session.cfda
          }
        ]);
      }
    }
    this.debug('Call Failed');
    this.session.call_failed = true;
    this.notify({
      state: 'failed'
    });
  };

}).call(this);
