// Generated by CoffeeScript 2.3.1
(function() {
  var debug, pkg;

  pkg = require('../../package.json');

  this.name = `${pkg.name}:middleware:client:setup`;

  debug = (require('tangible'))(this.name);

  // * doc.local_number Record with an identifier `number:<local-number>@<number-domain>`. These records are used on the client-side SBCs. They are one of the two types of `doc.number`.
  // * doc.number If the identifier of a number contains a `@` character, it is a `doc.local_number`.
  // * doc.local_number._id (required) `number:<local-number>@<number-domain>`
  // * doc.local_number.type (required) `number`
  // * doc.local_number.number (required) `<local-number>@<number-domain>`

  // Config
  // ======

  // Note that client-side the fields are called `profiles` and are stored in the JSON configuration.

  // Call-Processing
  // ===============
  this.include = async function() {
    var Reference, base, base1, base2, base3, base4, base5, base6, base7, m, our_dialplan, p, ref, ref1, ref2, ref3, ref4, ref5, ref6, reference_in, referred_by, sip_params, sip_profile_carrier, sip_profile_client, sip_refer_to, sip_referred_by;
    // Session Context
    // ---------------

    // For example, in `@data` we might get:

    // ```
    // 'Channel-Context': 'sbc-ingress',
    // variable_sofia_profile_name: 'huge-play-sbc-ingress',
    // variable_recovery_profile_name: 'huge-play-sbc-ingress',
    // ```

    // The `sofia_profile_name` above is the one for the inbound leg (`A` leg). For the outbound leg we use the profile "on the other side"; its name is stored in  @session.sip_profile .

    // * session.context (string) The original Sofia Context for this (ingress) call.

    // For calls originating internally, module `exultant-songs` will use the `origination_context` variable.
    // We load it first because otherwise the `Channel-Context` value (`default`) set by originate will take precedence.
    if ((base = this.session).context == null) {
      base.context = this.req.variable('origination_context');
    }
    // The channel-context is set (for calls originating from sofia-sip) by the `context` parameter of the Sofia instance that carries the A leg.
    if ((base1 = this.session).context == null) {
      base1.context = this.data['Channel-Context'];
    }
    this.debug('>>>> New call', this.session.context, this.data);
    if (!(m = (ref = this.session.context) != null ? ref.match(/^(\S+)-(ingress|egress|transfer|handled)(?:-(\S+))?$/) : void 0)) {
      this.debug.dev('Malformed context', this.session.context);
      return this.respond('500 Malformed context');
    }
    this.session.profile = m[1];
    this.direction(m[2]);
    if ((base2 = this.session).reference == null) {
      base2.reference = m[3];
    }
    // Session Reference
    // -----------------

    // * session.reference (string) Identifies a call spanning multiple FreeSwitch servers.

    // The `reference` is used to track a given call through various systems and associate parameters (e.g. client information) to the call as a whole.
    // In case of a transfer, the session identifier might be included in the context.
    // In case of a call from `exultant-songs`, the session identifier is in variable `session_reference`.
    if ((base3 = this.session).reference == null) {
      base3.reference = this.req.variable('session_reference');
    }
    // In all other cases, look (very hard) for a `xref` parameter.
    reference_in = (name) => {
      var base4, v;
      v = this.req.variable(name);
      if (v == null) {
        return;
      }
      m = v.match(/xref[=:]([\w-]+)/);
      if ((m != null) && (m[1] != null)) {
        return (base4 = this.session).reference != null ? base4.reference : base4.reference = m[1];
      } else {
        return this.debug(`${name} ${v} has no xref`);
      }
    };
    reference_in('sip_from_params');
    reference_in('sip_to_params');
    reference_in('sip_req_params');
    reference_in('sip_contact_params');
    reference_in('sip_referred_by_params');
    reference_in('sip_h_X-FS-Refer-Params');
    // If the call was originated by a phone it's normal for it to not have a reference,
    // in which case one is created.
    ({Reference} = this.cfg);
    this.reference = new Reference(this.session.reference);
    this.session.reference = this.reference.id;
    // Logger
    // ------
    if ((await this.reference.get_dev_logger())) {
      this.session.dev_logger = true;
    }
    this.notify({
      state: 'incoming-call-client-side'
    });
    // SIP Profile
    // -----------

    // Define the (sofia-sip) SIP profiles used to send calls out.
    sip_profile_client = `${pkg.name}-${this.session.profile}-egress`;
    sip_profile_carrier = `${pkg.name}-${this.session.profile}-ingress`;
    if ((base4 = this.session).sip_profile_client == null) {
      base4.sip_profile_client = sip_profile_client;
    }
    if ((base5 = this.session).sip_profile_carrier == null) {
      base5.sip_profile_carrier = sip_profile_carrier;
    }
    if (this.session.direction === 'ingress') {
      if ((base6 = this.session).sip_profile == null) {
        base6.sip_profile = sip_profile_client;
      }
    } else {
      if ((base7 = this.session).sip_profile == null) {
        base7.sip_profile = sip_profile_carrier;
      }
    }
    this.session.transfer = false;
    // The default transfer context assumes the transfer request is coming from a customer (egress call) and the customer is transfering the call.
    this.session.default_transfer_context = [this.session.profile, 'transfer', this.session.reference].join('-');
    if (this.session.direction === 'transfer' || (this.req.variable('transfer_source') != null)) {
      this.session.transfer = true;
      // For a blind transfer (or an attended transfer to FIFO) we'll get:
      // ```
      // variable_transfer_source: "(unix epoch time):(new profile's uuid):bl_xfer:(destination)/sbc-transfer-(xref)/inline:'socket:… async full'"
      // variable_sip_h_Referred-By: '["name"] <sip:(referrer)@…;xref:(xref)>',
      // variable_sip_refer_to: '<sip:(new-dest)@…>',
      // ```

      // ```
      // Channel-Transfer-Source
      // Caller-Transfer-Source
      // variable_transfer_source
      // variable_transfer_history
      // variable_pre_transfer_caller_id_name
      // variable_pre_transfer_caller_id_number
      // ```
      sip_refer_to = this.req.variable('sip_refer_to');
      sip_referred_by = this.req.variable('sip_h_Referred-By');
      if ((sip_refer_to != null) && (sip_referred_by != null)) {
        this.debug('source', this.source);
        m = sip_refer_to.match(/sip:(\d+)@/);
        this.debug('refer_to', m[1]);
        this.session.refer_to = m[1];
        this.destination = m[1];
        m = sip_referred_by.match(/sip:(\d+@[^;]+)/);
        this.debug('referred_by', m[1]);
        referred_by = m[1];
        this.session.referred_by = m[1];
        this.session.wait_for_aleg_ack = false; // in huge-play
        this.session.sip_wait_for_aleg_ack = false; // in tough-rate
      }
      this.direction('egress');
    }
    // The handled transfer context assumes the transfer request is coming from a (presumably trusted) server. It is used by the tough-rate call-handler.
    // For now this should only happen when a customer calls a global number that points to a conference, and the server that handled the request isn't the one serving the conference.
    this.session.handled_transfer_context = [this.session.profile, 'handled', this.session.reference].join('-');
    if (this.session.direction === 'handled') {
      this.session.transfer = true;
      this.direction('handled');
    }
    // SIP Profile Data
    // ----------------

    // Note that client-side the fields are called `profiles` and are stored in the JSON configuration.

    // * session.local_server (string, host:port) URI domain-part usable for REFER, etc. so that other servers might redirect calls to us (client-side only).
    p = (ref1 = this.cfg.profiles) != null ? ref1[this.session.profile] : void 0;
    if (p != null) {
      this.session.local_server = `${this.cfg.host}:${(ref2 = p.ingress_sip_port) != null ? ref2 : p.sip_port}`;
      this.session.client_server = `${this.cfg.host}:${(ref3 = p.egress_sip_port) != null ? ref3 : p.sip_port + 10000}`;
    } else {
      this.debug.dev('Missing profile', this.session.profile);
    }
    // Set FreeSwitch variables
    // ------------------------
    sip_params = `xref=${this.session.reference}`;
    our_dialplan = `inline:'socket:127.0.0.1:${(ref4 = this.cfg.port) != null ? ref4 : 5702} async full'`;
    await this.set({
      session_reference: this.session.reference,
      force_transfer_context: this.session.default_transfer_context
    });
    // force_transfer_dialplan: our_dialplan
    await this.export({
      session_reference: this.session.reference,
      // Info for handling of 302 etc. for (I assume) our outbound calls. `cfg.port` is from `thinkable-ducks/server`.
      sip_redirect_profile: this.session.profile,
      sip_redirect_context: this.session.default_transfer_context,
      sip_redirect_dialplan: our_dialplan,
      sip_redirect_contact_params: sip_params,
      sip_invite_params: sip_params,
      sip_invite_to_params: sip_params,
      sip_invite_contact_params: sip_params,
      sip_invite_from_params: sip_params
    });
    this.debug('Ready', {
      reference: this.session.reference,
      call: this.call.uuid,
      session: this.session._id,
      context: this.session.context,
      direction: this.session.direction,
      destination: this.destination,
      source: this.source,
      transfer: this.session.transfer,
      profile: this.session.profile,
      sip_profile: this.session.sip_profile,
      default_transfer_context: this.session.default_transfer_context,
      wait_for_aleg_ack: (ref5 = this.session.wait_for_aleg_ack) != null ? ref5 : null,
      local_server: (ref6 = this.session.local_server) != null ? ref6 : null
    });
  };

}).call(this);
