// Generated by CoffeeScript 2.3.1
(function() {
  var debug, pkg, qs, request;

  pkg = require('../package.json');

  this.name = `${pkg.name}:middleware:prompt`;

  debug = (require('tangible'))(this.name);

  request = require('request');

  qs = require('querystring');

  this.web = function() {
    return this.cfg.versions[pkg.name] = pkg.version;
  };

  // Use `mod_httpapi` to support URLs.
  this.include = function() {
    var prompt;
    prompt = {};
    // `record`
    // ========

    // Record using the given file or uri.

    // https://wiki.freeswitch.org/wiki/Misc._Dialplan_Tools_record
    prompt.record = async(file, time_limit = 300) => {
      var body, duration, ref, res, silence_hits, silence_thresh;
      this.debug('record', {file, time_limit});
      silence_thresh = 20;
      silence_hits = 3;
      await this.action('set', 'playback_terminators=any');
      await this.action('gentones', '%(500,0,800)');
      res = (await this.action('record', [file, time_limit, silence_thresh, silence_hits].join(' ')));
      body = (ref = res != null ? res.body : void 0) != null ? ref : {};
      // The documentation says:
      // - record_ms
      // - record_samples
      // - playback_terminator_used
      duration = body.variable_record_seconds;
      this.debug('record', {duration});
      return duration;
    };
    // `play_and_get_digits`
    // =====================

    // Simple wrapper for FreeSwitch's `play_and_get_digits`.

    // Required options:
    // - `min`
    // - `max`
    // - `timeout`
    // - `file`
    // - `var_name`
    // - `regexp`
    // - `digit_timeout`

    // https://wiki.freeswitch.org/wiki/Misc._Dialplan_Tools_play_and_get_digits
    prompt.play_and_get_digits = (o) => {
      var ref, ref1, ref2, ref3;
      this.debug('play_and_get_digits', o);
      return this.action('play_and_get_digits', [o.min, o.max, (ref = o.tries) != null ? ref : 1, o.timeout, (ref1 = o.terminators) != null ? ref1 : '#', o.file, (ref2 = o.invalid_file) != null ? ref2 : 'silence_stream://250', o.var_name, o.regexp, o.digit_timeout, (ref3 = o.transfer_on_failure) != null ? ref3 : ''].join(' '));
    };
    // `play`
    // ======

    // Play a file and optionnally record a single digit.
    // Promise resolves into the selected digit or `null`.
    prompt.play = async function(file, o = {}) {
      var body, name, ref, ref1, res;
      o.file = file;
      if (o.min == null) {
        o.min = 1;
      }
      if (o.max == null) {
        o.max = 1;
      }
      if (o.timeout == null) {
        o.timeout = 1000;
      }
      if (o.var_name == null) {
        o.var_name = 'choice';
      }
      if (o.regexp == null) {
        o.regexp = '\\d';
      }
      if (o.digit_timeout == null) {
        o.digit_timeout = 1000;
      }
      res = (await prompt.play_and_get_digits(o));
      body = (ref = res != null ? res.body : void 0) != null ? ref : {};
      name = `variable_${o.var_name}`;
      debug(`Got ${body[name]} for ${name}`);
      return (ref1 = body[name]) != null ? ref1 : null;
    };
    // `get_choice`
    // ========

    // Play a file and optionnaly record a single digit.
    // Promise resolves into the selected digit or `null`.
    prompt.get_choice = function(file, o = {}) {
      if (o.timeout == null) {
        o.timeout = 15000;
      }
      if (o.digit_timeout == null) {
        o.digit_timeout = 3000;
      }
      return prompt.play(file, o);
    };
    // `get_number`
    // ============

    // Asks for a number.
    // Promise resolves into the number or `null`.
    prompt.get_number = function(o = {}) {
      if (o.file == null) {
        o.file = 'phrase:voicemail_enter_id:#';
      }
      if (o.invalid_file == null) {
        o.invalid_file = "phrase:'voicemail_fail_auth'";
      }
      if (o.min == null) {
        o.min = 1;
      }
      if (o.max == null) {
        o.max = 16;
      }
      if (o.var_name == null) {
        o.var_name = 'number';
      }
      if (o.regexp == null) {
        o.regexp = '\\d+';
      }
      if (o.digit_timeout == null) {
        o.digit_timeout = 3000;
      }
      return prompt.get_choice(o.file, o);
    };
    // `get_pin`
    // =========

    // Asks for a PIN.
    // Promise resolves into the PIN or `null`.
    prompt.get_pin = function(o = {}) {
      if (o.file == null) {
        o.file = 'phrase:voicemail_enter_pass:#';
      }
      if (o.min == null) {
        o.min = 4;
      }
      if (o.max == null) {
        o.max = 16;
      }
      if (o.var_name == null) {
        o.var_name = 'pin';
      }
      return prompt.get_number(o);
    };
    // `get_new_pin`
    // =============

    // Asks for a new PIN.
    // Promise resolves into the new PIN or `null`.
    prompt.get_new_pin = function(o = {}) {
      if (o.var_name == null) {
        o.var_name = 'new_pin';
      }
      o.invalid_file = 'silence_stream://250';
      return prompt.get_pin(o);
    };
    prompt.goodbye = async() => {
      this.debug('goodbye');
      await prompt.phrase('voicemail_goodbye');
      await this.action('hangup');
      return this.debug('goodbye done');
    };
    prompt.phrase = async(phrase) => {
      this.debug('phrase');
      return (await this.action('phrase', phrase));
    };
    prompt.error = async function(id) {
      debug('error', {id});
      if (id != null) {
        await prompt.phrase(`spell,${id}`);
      }
      await prompt.goodbye();
      return Promise.reject(new Error(`error ${id}`));
    };
    // `uri`
    // -----

    // Provide a URI to access the web services (attachment upload/download) defined below.
    // Note that since FreeSwitch has trouble with query parameters (`Invalid file format [wav?rev=…]`) the revision number has to be provided as part of the path.
    prompt.uri = (domain, db, id, file, rev) => {
      var host, path, port, ref;
      host = (ref = this.cfg.web.host) != null ? ref : '127.0.0.1';
      port = this.cfg.web.port;
      domain = qs.escape(domain);
      db = qs.escape(db);
      id = qs.escape(id);
      rev = qs.escape(rev != null ? rev : 'no-rev');
      file = qs.escape(file);
      path = `/${domain}/${db}/${id}/${rev}/${file}`;
      return `http://${host}:${port}${path}`;
    };
    this.prompt = prompt;
  };

  // Attachment upload/download
  // ==========================
  this.web = function() {
    var cfg, translators;
    cfg = this.cfg;
    this.translators = translators = {
      // A translator for user databases.
      'user-db': (db, id, file) => {
        db = qs.escape(db);
        id = qs.escape(id);
        file = qs.escape(file);
        return {
          base: cfg.userdb_base_uri,
          uri: `/${db}/${id}/${file}`
        };
      },
      // A translator for the local provisioning database (the `db` database name is ignored).
      'prov': (db, id, file) => {
        id = qs.escape(id);
        file = qs.escape(file);
        return {
          base: cfg.provisioning,
          uri: `${id}/${file}`
        };
      },
      'master-prov': (db, id, file) => {
        var masters;
        id = qs.escape(id);
        file = qs.escape(file);
        masters = cfg.prov_master_admin;
        if (masters != null) {
          if (typeof masters === 'string') {
            masters = [masters];
          }
        }
        return {
          base: masters[0],
          uri: `${id}/${file}`
        };
      }
    };
    this.get('/:domain/:db/:id/:rev/:file', function() {
      var base, proxy, translator, uri;
      translator = translators[this.params.domain];
      if (translator == null) {
        this.next('Invalid domain');
        return;
      }
      ({base, uri} = translator(this.params.db, this.params.id, this.params.file));
      proxy = request.get({
        baseUrl: base,
        uri: uri,
        followRedirects: false,
        maxRedirects: 0
      });
      this.request.pipe(proxy).on('error', (error) => {
        debug(`GET ${uri} : ${error}`);
        this.next(`GET ${uri} : ${error}`);
      });
      proxy.pipe(this.response);
    });
    return this.put('/:domain/:db/:id/:rev/:file', function() {
      var base, proxy, translator, uri;
      translator = translators[this.params.domain];
      if (translator == null) {
        this.next('Invalid domain');
        return;
      }
      ({base, uri} = translator(this.params.db, this.params.id, this.params.file));
      proxy = request.put({
        baseUrl: base,
        uri: uri,
        qs: {
          rev: this.params.rev
        },
        followRedirects: false,
        maxRedirects: 0,
        timeout: 120
      });
      this.request.pipe(proxy).on('error', (error) => {
        var ref;
        debug(`GET ${uri} rev ${this.params.rev} : ${(ref = error.stack) != null ? ref : error}`);
        try {
          this.res.end();
        } catch (error1) {}
      });
      proxy.pipe(this.response);
    });
  };

}).call(this);
